@@ -1,18 +1,18 @@
 #
-# Command line tools for Myokit.
+# Command line tools.
 #
-# This file is part of Myokit
-#  Copyright 2011-2018 Maastricht University, University of Oxford
-#  Licensed under the GNU General Public License v3.0
-#  See: http://myokit.org
+# This file is part of Myokit.
+# See http://myokit.org for copyright, sharing, and licensing details.
 #
 from __future__ import absolute_import, division
 from __future__ import print_function, unicode_literals
 
 
+_line_width = 79
+
+
 def printline():
-    line_width = 60
-    print('-' * line_width)
+    print('-' * _line_width)
 
 
 def main():
@@ -71,7 +71,7 @@ def main():
         func(**args)
 
 
-def block(filename, pyqt4=False, pyqt5=False, pyside=False):
+def block(filename, pyqt4=False, pyqt5=False, pyside=False, pyside2=False):
     """
     Runs the DataBlock viewer.
     """
@@ -80,17 +80,25 @@ def block(filename, pyqt4=False, pyqt5=False, pyside=False):
         myokit.FORCE_PYQT5 = True
         myokit.FORCE_PYQT4 = False
         myokit.FORCE_PYSIDE = False
+        myokit.FORCE_PYSIDE2 = False
     elif pyqt4:
         myokit.FORCE_PYQT5 = False
         myokit.FORCE_PYQT4 = True
         myokit.FORCE_PYSIDE = False
+        myokit.FORCE_PYSIDE2 = False
     elif pyside:
-        myokit.FORCE_PYQT4 = False
+        myokit.FORCE_PYQT5 = False
         myokit.FORCE_PYQT4 = False
         myokit.FORCE_PYSIDE = True
+        myokit.FORCE_PYSIDE2 = False
+    elif pyside2:
+        myokit.FORCE_PYQT5 = False
+        myokit.FORCE_PYQT4 = False
+        myokit.FORCE_PYSIDE = False
+        myokit.FORCE_PYSIDE2 = True
     import myokit.gui
     import myokit.gui.datablock_viewer
-    if pyqt5 or pyqt4 or pyside:
+    if pyqt5 or pyqt4 or pyside or pyside2:
         print('Using backend: ' + myokit.gui.backend)
     myokit.gui.run(myokit.gui.datablock_viewer.DataBlockViewer, filename)
 
@@ -126,6 +134,11 @@ def add_block_parser(subparsers):
         action='store_true',
         help='Run the DataBlock Viewer using the PySide backend.',
     )
+    parser.add_argument(
+        '--pyside2',
+        action='store_true',
+        help='Run the DataBlock Viewer using the PySide2 backend.',
+    )
     parser.set_defaults(func=block)
 
 
@@ -166,12 +179,12 @@ def add_compare_parser(subparsers):
     parser.set_defaults(func=compare)
 
 
-def compiler():
+def compiler(debug):
     """
     Tests for C compilation support.
     """
     import myokit
-    compiler = myokit.Compiler.info()
+    compiler = myokit.Compiler.info(debug)
     if compiler is None:
         print('Compilation with distutils/setuptools failed.')
     else:
@@ -187,6 +200,11 @@ def add_compiler_parser(subparsers):
         description='Checks for C compilation support.',
         help='Prints information about C compilation support.',
     )
+    parser.add_argument(
+        '--debug',
+        action='store_true',
+        help='Show error output.',
+    )
     parser.set_defaults(func=compiler)
 
 
@@ -275,7 +293,8 @@ def mmt_export(exporter, source, target):
     import myokit.formats
 
     # Get exporter
-    exporter = myokit.formats.exporter(exporter)
+    name = exporter
+    exporter = myokit.formats.exporter(name)
 
     # Set to auto-print
     logger = exporter.logger()
@@ -301,7 +320,10 @@ def mmt_export(exporter, source, target):
     if exporter.supports_model():
         # Export model
         logger.log('Exporting model')
-        exporter.model(target, model)
+        if name == 'cellml':
+            exporter.model(target, model, protocol)
+        else:
+            exporter.model(target, model)
     else:
         # Export runnable
         logger.log('Exporting runnable')
@@ -453,7 +475,10 @@ def install_gnome_kde():
         if template:
             # Process templates, create files
             p = myokit.pype.TemplateEngine()
-            varmap = {'icons': dir_icons}
+            varmap = {
+                'icons': dir_icons,
+                'python': sys.executable,
+            }
             with open(dest, 'w') as f:
                 p.set_output_stream(f)
                 p.process(orig, varmap)
@@ -485,6 +510,7 @@ def install_gnome_kde():
     path = os.path.join(home, '.local', 'share', 'mime', 'packages')
     name = 'x-wcp.xml'
     place_file(path, name)
+
     # Reload mime database
     print('Reloading mime database')
     path = home + '/.local/share/mime/'
@@ -510,12 +536,12 @@ def install_windows():
         raise Exception('Not a windows machine.')
 
     import os
-    #import shutil
     import tempfile
 
+    import menuinst
+
     import myokit
     import myokit.pype
-    import menuinst
 
     # Process template to get icon directory
     tdir = tempfile.mkdtemp()
@@ -534,8 +560,7 @@ def install_windows():
         print('Done')
 
     finally:
-        #shutil.rmtree(tdir)
-        pass
+        myokit._rmtree(tdir)
 
 
 def add_icon_parser(subparsers):
@@ -550,7 +575,7 @@ def add_icon_parser(subparsers):
     parser.set_defaults(func=install)
 
 
-def ide(filename, pyqt4=False, pyqt5=False, pyside=False):
+def ide(filename, pyqt4=False, pyqt5=False, pyside=False, pyside2=False):
     """
     Runs the Myokit IDE.
     """
@@ -559,17 +584,25 @@ def ide(filename, pyqt4=False, pyqt5=False, pyside=False):
         myokit.FORCE_PYQT5 = True
         myokit.FORCE_PYQT4 = False
         myokit.FORCE_PYSIDE = False
+        myokit.FORCE_PYSIDE2 = False
     elif pyqt4:
         myokit.FORCE_PYQT5 = False
         myokit.FORCE_PYQT4 = True
         myokit.FORCE_PYSIDE = False
+        myokit.FORCE_PYSIDE2 = False
     elif pyside:
-        myokit.FORCE_PYQT4 = False
+        myokit.FORCE_PYQT5 = False
         myokit.FORCE_PYQT4 = False
         myokit.FORCE_PYSIDE = True
+        myokit.FORCE_PYSIDE2 = False
+    elif pyside2:
+        myokit.FORCE_PYQT5 = False
+        myokit.FORCE_PYQT4 = False
+        myokit.FORCE_PYSIDE = False
+        myokit.FORCE_PYSIDE2 = True
     import myokit.gui
     import myokit.gui.ide
-    if pyqt5 or pyqt4 or pyside:
+    if pyqt5 or pyqt4 or pyside or pyside2:
         print('Using backend: ' + myokit.gui.backend)
     myokit.gui.run(myokit.gui.ide.MyokitIDE, filename)
 
@@ -605,6 +638,11 @@ def add_ide_parser(subparsers):
         action='store_true',
         help='Run the IDE using the PySide backend.',
     )
+    parser.add_argument(
+        '--pyside2',
+        action='store_true',
+        help='Run the DataBlock Viewer using the PySide2 backend.',
+    )
     parser.set_defaults(func=ide)
 
 
@@ -626,17 +664,30 @@ def mmt_import(importer, source, target=None):
     logger.log_flair(str(importer.__class__.__name__))
 
     # Import
-    m = importer.model(source)
+    model = importer.model(source)
+
+    # Print any warnings
+    logger.log_warnings()
+
+    # Try to split off an embedded protocol
+    protocol = myokit.lib.guess.remove_embedded_protocol(model)
+
+    # No protocol? Then create one
+    if protocol is None:
+        protocol = myokit.default_protocol(model)
+
+    # Get default script
+    script = myokit.default_script(model)
 
     # If a target is specified, save the output
     if target:
         # Save or output model to new location
         logger.log('Saving output to ' + str(target))
-        myokit.save(target, m)
+        myokit.save(target, model, protocol, script)
         logger.log('Done.')
     else:
         # Write it to screen
-        print(myokit.save(None, m))
+        print(myokit.save(None, model, protocol, script))
 
 
 def add_import_parser(subparsers):
@@ -862,8 +913,7 @@ def reset(force=False):
     if remove:
         print('Removing')
         print('  ' + myokit.DIR_USER)
-        import shutil
-        shutil.rmtree(myokit.DIR_USER)
+        myokit._rmtree(myokit.DIR_USER)
         print('Done')
     else:
         print('Aborting.')
@@ -1026,7 +1076,7 @@ def step(source, ref, ini, raw):
             print('Reading model from ' + source + '...')
         model = myokit.load_model(source)
         if not raw:
-            print('Model ' + model.name() + ' read successfully.')
+            print('Model ' + source + ' read successfully.')
     except myokit.ParseError as ex:
         print(myokit.format_parse_error(ex, source))
         sys.exit(1)
@@ -1044,7 +1094,7 @@ def step(source, ref, ini, raw):
             print(myokit.step(model, initial=ini, reference=ref))
     except myokit.NumericalError as ee:
         e = 'Numerical error'
-        n = line_width - len(e) - 2
+        n = _line_width - len(e) - 2
         print('-' * int(n / 2) + ' ' + e + ' ' + '-' * (n - int(n / 2)))
         print('A numerical error occurred:')
         print(str(ee))
